require 'wolverine/script'
require 'wolverine/lua_error'

module Roundhouse
  # Modifies the Shopify Wolverine library. The Roundhouse::Script will
  # take a string and load it up to redis directly. No method-missing, no
  # autoloading, no templating.
  # Optionally pass a :name, so we can collect stats. Otherwise it will be the SHA1 hash
  class Script < Wolverine::Script
    def initialize content, options = {}
      @content = content
      @digest  = Digest::SHA1.hexdigest @content
      @config  = options[:config] || fail('Must supply a Wolverine::Configuration object')
      @name    = options[:name] || @digest
    end

    def statsd
      "Wolverine.#{@name}"
    end

    def instrument eval_type
      ret = nil
      runtime = Benchmark.realtime { ret = yield }
      @config.instrumentation.call @name, runtime, eval_type
      ret
    end

    class Configuration
      attr_reader :instrumentation

      def initialize(&blk)
        @instrumentation = blk || proc { |script_name, runtime, eval_type| Roundhouse.logger.info "REDIS LUA [#{eval_type}]: #{script_name} #{runtime}" }
      end
    end
  end
end

# Mock the Wolverine class
class Wolverine
  class << self
    # If we added Roundhouse into a Rails app and we fully configure StatsD and
    # Wolverine, then we don't want this proxy to interfere
    unless defined?(statsd_enabled?)
      def statsd_enabled?
        false
      end
    end
  end
end
